
import { AxiosRequestTransformer, AxiosResponseTransformer } from "axios"
import { ApiResponseError, RawResponse } from "./Types"

/**
 * 
 * @param data request data
 * @param headers request headers
 * @returns 
 */
export const jsonRequest: AxiosRequestTransformer = (data, headers) => {
  if (typeof data !== 'string') {
    /**
     * Stringifies the data and sets a header if data is not a string
     */
    data = JSON.stringify(data)
    headers.setContentType('application/json')
  }
  console.log('jsonRequest', { data })
  return data
}

/**
 * 
 * @param data response data
 * @param headers response headers
 * @returns properly parsed data
 */
export const jsonResponse: AxiosResponseTransformer = (data, headers) => {
  try {
    data = JSON.parse(data)
  } catch (error) {
  }
  // console.log('jsonResponse', { data })
  return data
}

/**
 * 
 * @param response the raw AxiosResponse which contains a `data` property
 * @returns Just the data, properly typed as T & {error, errors}
 */
export function responseHandler<T = unknown>(response: RawResponse<T>): T & ApiResponseError {
  let { data } = response
  // console.log('responseHandler', { data })
  if (data.error) console.error(data.error)
  if (data.errors) console.error(data.errors)
  return data
}

